package com.chelvc.cloud.vehicle.server.service.impl;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.dto.EmployeeDTO;
import com.chelvc.cloud.uc.api.dto.UserDTO;
import com.chelvc.cloud.uc.api.model.Scope;
import com.chelvc.cloud.uc.api.service.EmployeeService;
import com.chelvc.cloud.uc.api.service.UserService;
import com.chelvc.cloud.vehicle.api.constant.ReserveStatus;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.dto.ReservationDTO;
import com.chelvc.cloud.vehicle.api.param.ReservationModifyParam;
import com.chelvc.cloud.vehicle.api.param.ReservationPagingParam;
import com.chelvc.cloud.vehicle.api.param.ReservationQueryParam;
import com.chelvc.cloud.vehicle.server.copier.ReservationCopier;
import com.chelvc.cloud.vehicle.server.dao.ReservationMapper;
import com.chelvc.cloud.vehicle.server.entity.Reservation;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.cloud.vehicle.server.service.ReservationService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

/**
 * 预约记录业务操作实现
 *
 * @author Woody
 * @date 2023/7/17
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.ReservationService.class)
public class ReservationServiceImpl extends ServiceImpl<ReservationMapper, Reservation> implements ReservationService,
        com.chelvc.cloud.vehicle.api.service.ReservationService {

    private final MerchantService merchantService;

    private final UserService userService;

    @Override
    public void addReservation(@NonNull Long merchantId, @NonNull ReservationModifyParam param) {
        Reservation reservation = ReservationCopier.INSTANCE.copying(param);
        reservation.setUserId(SessionContextHolder.getId());
        reservation.setMerchantId(merchantId);
        reservation.setStatus(ReserveStatus.PENDING);
        this.save(reservation);
    }

    @Override
    public List<ReservationDTO> listUserReservations(@NonNull ReservationQueryParam param) {
        Long userId = SessionContextHolder.getId();
        List<Reservation> reservations = this.baseMapper.listUserReservations(userId, param);
        if (CollectionUtils.isEmpty(reservations)) {
            return Collections.emptyList();
        }
        return reservations
                .stream()
                .map(o -> {
                    ReservationDTO reservationDTO = ReservationCopier.INSTANCE.copying(o);
                    reservationDTO.setMerchant(o.getMerchant());
                    return reservationDTO;
                }).collect(Collectors.toList());

    }

    @Override
    public void deleteReservation(Long id) {
        this.baseMapper.deleteById(id);
    }

    @Override
    public Pagination<ReservationDTO> getReservationPaging(ReservationPagingParam param) {
        Long userId = SessionContextHolder.getId();
        Scope scope = StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
            Page<Reservation> page = this.lambdaQuery()
                    .eq(StringUtils.notEmpty(param.getStatus()), Reservation::getStatus, param.getStatus())
                    .orderByDesc(Reservation::getAppointTime)
                    .page(DatabaseContextHolder.page(param.getPaging()));
            Pagination<ReservationDTO> pagination = DatabaseContextHolder.pagination(page,
                    ReservationCopier.INSTANCE::copying);
            pagination.getRecords().forEach(item -> {
                UserDTO user = this.userService.getUser(item.getUserId());
                if (user != null) {
                    item.setUsername(user.getNickname());
                }
                MerchantDTO merchantDTO = this.merchantService.getMerchantById(item.getMerchantId());
                item.setMerchant(merchantDTO);
            });
            return pagination;
        } else {
            List<Long> merchants = this.merchantService.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            Page<Reservation> page = this.lambdaQuery()
                    .eq(StringUtils.notEmpty(param.getStatus()), Reservation::getStatus, param.getStatus())
                    .in(Reservation::getMerchantId, merchants)
                    .orderByDesc(Reservation::getAppointTime)
                    .page(DatabaseContextHolder.page(param.getPaging()));
            Pagination<ReservationDTO> pagination = DatabaseContextHolder.pagination(page,
                    ReservationCopier.INSTANCE::copying);
            pagination.getRecords().forEach(item -> {
                UserDTO user = this.userService.getUser(item.getUserId());
                if (user != null) {
                    item.setUsername(user.getNickname());
                }
                MerchantDTO merchantDTO = this.merchantService.getMerchantById(item.getMerchantId());
                item.setMerchant(merchantDTO);
            });
            return pagination;
        }
    }


    @Override
    public List<ReservationDTO> listMerchantReservations(@NonNull ReservationQueryParam param) {
        if (StringUtils.isEmpty(param.getMerchantId())) {
            return null;
        }
        List<Reservation> list = this.lambdaQuery()
                .eq(StringUtils.notEmpty(param.getStatus()), Reservation::getStatus, param.getStatus())
                .in(Reservation::getMerchantId, param.getMerchantId())
                .orderByDesc(Reservation::getAppointTime)
                .list();
        List<ReservationDTO> reservations = ReservationCopier.INSTANCE.copying(list);
        reservations.forEach(item -> {
            UserDTO user = this.userService.getUser(item.getUserId());
            if (user != null) {
                item.setUsername(user.getNickname());
            }
            MerchantDTO merchantDTO = this.merchantService.getMerchantById(item.getMerchantId());
            item.setMerchant(merchantDTO);
        });
        return reservations;
    }
    @Override
    public void updateReservationsStatus(Long id,ReserveStatus status){
        Reservation reservation = AssertUtils.available(this.getById(id), "该预约单不存在");
        reservation.setStatus(status);
        this.updateById(reservation);
    }
    @Override
    public ReservationDTO getDetail(Long id){
        Reservation reservation = this.getById(id);
        MerchantDTO merchant = this.merchantService.getMerchantById(reservation.getMerchantId());
        ReservationDTO reservationDTO = ReservationCopier.INSTANCE.copying(reservation);
        UserDTO user = this.userService.getUser(reservation.getUserId());
        if (user != null) {
            reservationDTO.setUsername(user.getNickname());
        }
        if (null != reservationDTO){
            reservationDTO.setMerchant(merchant);
        }
        return reservationDTO;
    }
}
